import State
import Data.Word
import Data.Int
import Data.Bits
import Data.Text.Encoding

data TypeExpression = 
    TBoolean
    | TChar
    | TInt
    | TReal
    | TPointer
    | TRecord [TypeExpression]
    | TArray TypeExpression Int

data PrimitiveContainer =
    PrimitiveBool Bool 
    | PrimitiveChar Char 
    | PrimitiveInt16 Int16 
    | PrimitiveWord16 Word16    

toWord8 :: PrimitiveContainer -> [Word8]
toWord8 (PrimitiveBool True) = [1] 
toWord8 (PrimitiveBool False) = [0] 
toWord8 (PrimitiveInt16 x) = [fromIntegral $ (shiftR x 8) .&. 0xff, fromIntegral $ x .&. 0xff]
toWord8 (PrimitiveWord16 x) = [fromIntegral $ (shiftR x 8) .&. 0xff, fromIntegral $ x .&. 0xff]
toWord8 (PrimitiveChar ch) = [a, b, c, d]
    where
        int_ch = fromEnum ch 
        a = fromIntegral $ (shiftR int_ch 24) .&. 0xff
        b = fromIntegral $ (shiftR int_ch 16) .&. 0xff
        c = fromIntegral $ (shiftR int_ch 8) .&. 0xff
        d = fromIntegral $  int_ch .&. 0xff

sizeof :: TypeExpression -> MemorySize
sizeof TBoolean = 1
sizeof TChar = 4
sizeof TInt = 2
sizeof TReal = 4
sizeof TPointer = 2
sizeof (TRecord types) =  toInteger $ foldl (+) 0 $ map (fromIntegral . sizeof) types
sizeof (TArray basic size) = toInteger $ size * (fromIntegral $ sizeof basic)

